package skip;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertEquals;
import org.junit.Test;

import java.util.NoSuchElementException;
import java.util.Iterator;
import java.util.ArrayList;

public class SkipEachOtherIteratorTest {
    public static final ArrayList<Integer> EMPTY_LIST =
        new ArrayList<Integer>();
    
    public static SkipEachOtherIterator<Integer> makeEmptyIterator() {
        return new SkipEachOtherIterator<Integer>(EMPTY_LIST.iterator());
    }
                                                  
    @Test
    public void testDoesNotHaveNextOnEmpty() {
        final Iterator<Integer> it = makeEmptyIterator();
        assertFalse(it.hasNext());
    }

    @Test(expected = NoSuchElementException.class)
    public void testNextOnEmptyThrowsException() {
        final Iterator<Integer> it = makeEmptyIterator();
        it.next();
    }

    @Test
    public void singletonIteratorWorksCorrectly() {
        final ArrayList<Integer> numbers = new ArrayList<Integer>();
        numbers.add(new Integer(1));
        
        final Iterator<Integer> it =
            new SkipEachOtherIterator<Integer>(numbers.iterator());
        assertTrue(it.hasNext());
        assertEquals(it.next().intValue(), 1);
        assertFalse(it.hasNext());
    }

    @Test
    public void pairIteratorWorksCorrectly() {
        final ArrayList<Integer> numbers = new ArrayList<Integer>();
        numbers.add(new Integer(1));
        numbers.add(new Integer(2));
        
        final Iterator<Integer> it =
            new SkipEachOtherIterator<Integer>(numbers.iterator());
        assertTrue(it.hasNext());
        assertEquals(it.next().intValue(), 1);
        assertFalse(it.hasNext());
    }

    @Test
    public void tripleIteratorWorksCorrectly() {
        final ArrayList<Integer> numbers = new ArrayList<Integer>();
        numbers.add(new Integer(1));
        numbers.add(new Integer(2));
        numbers.add(new Integer(3));
        
        final Iterator<Integer> it =
            new SkipEachOtherIterator<Integer>(numbers.iterator());
        assertTrue(it.hasNext());
        assertEquals(it.next().intValue(), 1);
        assertTrue(it.hasNext());
        assertEquals(it.next().intValue(), 3);
        assertFalse(it.hasNext());
    }
}
